(** Diese Unit enthlt die Komponente TRecentFileList

@author Daniel Biehl
@version 1.0 $Id: RecentList.pas,v 1.9 1998/07/02 10:48:06 daniel_biehl Exp daniel_biehl $
*)
Unit RecentList;
{
$Log: RecentList.pas,v $
Revision 1.9  1998/07/02 10:48:06  daniel_biehl
GetCaption Korrigiert

Revision 1.8  1998/07/01 18:03:34  daniel_biehl
Icon Hinzugefgt

Revision 1.7  1998/07/01 17:02:40  daniel_biehl
Rechtschreibfehler korrigiert

Revision 1.6  1998/07/01 16:54:02  daniel_biehl
Letzte Doku ein gefgt

Revision 1.5  1998/07/01 16:46:36  daniel_biehl
Dokomentation hinzgefgt

Revision 1.4  1998/07/01 15:28:52  daniel_biehl
Initial Counter hinzugefgt
Erster vollstndige Version

Revision 1.3  1998/07/01 15:10:15  daniel_biehl
<>

Revision 1.2  1998/07/01 15:08:16  daniel_biehl
Configstring hinzugefgt

Revision 1.1  1998/07/01 14:29:36  daniel_biehl
Initial revision
}

Interface

Uses
  SysUtils, Classes, Menus;

Type
  (**Der Typ TRecentFileListMode wird fr die Eigenschaft Mode der
     Komponete TRecentFileList verwendet.

     Folgende Werte fr den Typ sind mglich:

     rflBefore - fgt das Menu vor dem mit MenuItem definierten Menu zu.
     rflAfter - fgt das Menu nach dem mit MenuItem definierten Menu zu.
     rflSubMenu - erzeugt ein Untermenu.

     @see Mode
   *)
  TRecentFileListMode = (rflBefore, rflAfter, rflSubMenu);
  (**Der Typ TRecentFileListOnClickEvent wird fr Ereignisse der Komponente
     TRecentFileList verwendet.

     @see OnMenuClick
   *)
  TRecentFileListOnClickEvent = Procedure(Sender: TObject; Filename: String) Of Object;

  (**TRecentFileList ist eine Komponente die eine Liste verwaltet in der die
     zuletzt bearbeiteten Dateien abgespeichert werden knnen.

     TRecentFileList fgt autmatisch die dazugehrigen Menueintrge in ein
     vorher definiertes Menu ein.
   *)
  TRecentFileList = Class(TComponent)
  Private
    { Private-Deklarationen }
    FList: TStringList;
    FMenuList: TList;

    FMenuItem: TMenuItem;
    FMaxEntries: integer;
    FMode: TRecentFileListMode;
    FInitialCounter: integer;
    FOnMenuClick: TRecentFileListOnClickEvent;

    Procedure SetMenuItem(value: TMenuItem);
    Procedure SetMaxEntries(Value: integer);
    Procedure SetMode(Value: TRecentFileListMode);
    Procedure SetInitialCounter(Value: integer);

    Function GetCaption(i: integer): String;
    Function AddMenu: TMenuItem;
    Procedure OnMenuItemClick(Sender: TObject);
    Function GetConfigString: String;
    Procedure SetConfigString(c: String);

    Procedure CreateMenu;
    Procedure FreeMenu;
    Procedure UpdateMenu;
    Procedure DoClick(Item: TMenuItem);


  Public
    { Public-Deklarationen }
    (**Die Methode Create erzeugt eine mit Standardwerten initialisierte Instanz
       einer Komponente und weist ihr ausreichend Speicher zu.
     *)
    Constructor Create(AOwner: TComponent); Override;
    (**Die Methode Destroy gibt die Komponente und die ihr untergeordneten
      Komponenten frei.
     *)
    Destructor Destroy; Override;
    (**Die Methode AddFile fgt eine Datei zu RecentFileList hinzu.
     *)
    Procedure AddFile(Filename: String);
    (**Die Eigenschaft Filelist ist ein Zeiger auf die Dateiliste.
      FileList bitte nur Lesend verwenden.
      @param Filename Datei die zu RecentFileList hinzugefgt werden soll
     *)
    Property FileList: TStringList Read FList;
    (**Die Eigenschaft ConfigString enthlt einen String der die Konfiguration
      der RecentFileList enthlt. Mit dieser Eigenschaft ist es mglich die
      RecentFileList zu Lesen und widerherszustellen.
     *)
    Property ConfigString: String Read GetConfigString Write SetConfigString;
  Published
    { Published-Deklarationen }
    (**Die Eigenschaft MenuItem enthlt das MenuItem zu dem das Menu hinzugefgt werden soll.
       @see MaxEntries
       @see InitialCounter
       @see Mode
     *)
    Property MenuItem: TMenuItem Read FMenuitem Write SetMenuItem;
    (**Mit der Eigenschaft MaxEntries kann die maximale Anzahl der Menueintrge festgelegt werden.
       Standardwert ist 5.

       @see MenuItem
     *)
    Property MaxEntries: integer Read FMaxEntries Write SetMaxEntries Default 5;
    (**Mit der Eigenschft Mode kann festgelegt werden wie die Menuitems hinzgefgt werden sollen.

       rflBefore - fgt das Menu vor dem mit MenuItem definierten Menu zu.
       rflAfter - fgt das Menu nach dem mit MenuItem definierten Menu zu.
       rflSubMenu - erzeugt ein Untermenu.

       Standardwert ist rflSubMenu.

       @see TRecentFileListMode
    *)
    Property Mode: TRecentFileListMode Read FMode Write SetMode Default rflSubMenu;
    (**Mit der Eigenschaft InitialCounter wird der Startwert festgelegt
       der im Menu als ShortCut angezeigt werden soll.
       Standardwert ist 0.

       @see MenuItem
     *)
    Property InitialCounter: integer Read FInitialCounter Write SetInitialCounter Default 0;
    (**Das Ereignis OnMenuClick tritt ein wenn einer der Eintrge in der RencentFileList klickt.

       TRecentFileListOnClickEvent = Procedure(Sender: TObject; Filename: String) Of Object;

       In einer Ereignisbehandlungsroutine fr OnClick knnen Aktionen implementieren werden,
       die ausgefhrt werden wenn ein Menueintrag der RecentFileList ausgewhlt wird.
       Der Parameter Filename gibt den Eintrag in der RecentFileList an.

       @see TRecentFileListOnClickEvent
       @see MenuItem
     *)
    Property OnMenuClick: TRecentFileListOnClickEvent Read FOnMenuClick Write FOnMenuClick;
  End;

Procedure register;

Implementation

Procedure register;
Begin
  RegisterComponents('dabiSoft', [TRecentFileList]);
End;

Constructor TRecentFileList.Create(AOwner: TComponent);
Begin
  Inherited Create(AOwner);
  FMaxEntries := 5;
  FList := TStringList.Create;
  FMenuList := TList.Create;
  FMode := rflSubMenu;
  FInitialCounter := 0;
End;

Destructor TRecentFileList.Destroy;
Begin
  FList.Free;
  FMenuList.Free;
  Inherited Destroy;
End;

Procedure TRecentFileList.AddFile(Filename: String);
Var
  i                                     : integer;
Begin
  i := FList.IndexOf(Filename);
  Case FList.IndexOf(Filename) Of
    - 1: FList.Insert(0, Filename);
    0: ;
  Else
    Begin
      Flist.Delete(i);
      FList.Insert(0, Filename);
    End;
  End;
  While Flist.Count > FMaxEntries Do
    Flist.Delete(FList.Count - 1);
  updateMenu;
End;

Function TRecentFileList.AddMenu: TMenuItem;
Begin
  result := Nil;
  If Assigned(FMenuItem) Then
  Begin
    result := TMenuItem.Create(Owner);
    Case FMode Of
      rflBefore:
        Begin
          If Assigned(FMenuItem.Parent) Then
          Begin
            FMenuItem.Parent.Insert(FMenuItem.MenuIndex, result);
          End;
        End;
      rflAfter:
        Begin
          If Assigned(FMenuItem.Parent) Then
          Begin
            If FMenuList.Count = 0 Then
              FMenuItem.Parent.Insert(FMenuItem.MenuIndex + 1, result)
            Else
            Begin
              TMenuItem(FMenuList[FMenuList.Count - 1]).Parent.Insert(TMenuItem(FMenuList[FmenuList.Count -
                1]).MenuIndex + 1, result);
            End;
          End;
        End;
      rflSubMenu: FMenuItem.Add(result);
    End;
    FMenuList.Add(result);
    result.OnClick := OnMenuItemClick;
  End;
End;

Function TRecentFileList.GetCaption(i: integer): String;
Begin
  result := '&' + intToHex(i + FInitialCounter,1) + ' ' + FList[i];
End;

Procedure TRecentFileList.SetInitialCounter(Value: integer);
Begin
  If Value <> FInitialCounter Then
  Begin
    FInitialCounter := Value;
    UpdateMenu;
  End;
End;

Procedure TRecentFileList.SetMenuItem(value: TMenuItem);
Begin
  If Value <> FMenuItem Then
  Begin
    FMenuITem := Value;
    CreateMenu;
  End;
End;

Procedure TRecentFileList.SetMaxEntries(Value: integer);
Begin
  If Value <> FMaxEntries Then
  Begin
    FMaxEntries := Value;
    UpdateMenu;
  End;
End;

Procedure TRecentFileList.SetMode(Value: TRecentFileListMode);
Begin
  If Value <> FMode Then
  Begin
    Fmode := Value;
    CreateMenu;
  End;
End;

Procedure TRecentFileList.CreateMenu;
Var
  i                                     : integer;
Begin
  Freemenu;
  If Assigned(FMenuItem) Then
    For i := 0 To FList.Count - 1 Do
    Begin
      With AddMenu Do
      Begin
        Caption := GetCaption(i);
        Tag := i;
      End;
    End;
End;

Procedure TRecentFileList.FreeMenu;
Var
  i                                     : integer;
Begin
  For i := 0 To FMenuList.Count - 1 Do
  Begin
    If Assigned(FMenuList[i]) Then TMenuItem(FMenuList[i]).Free;
  End;
  FMenuList.Clear;
End;

Procedure TRecentFileList.UpdateMenu;
Var
  i                                     : integer;
Begin
  If Assigned(FMenuItem) Then
  Begin
    For i := 0 To FList.Count - 1 Do
    Begin
      If FMenuList.Count > i Then
      Begin
        With TMenuItem(FMenuList[i]) Do
        Begin
          Caption := GetCaption(i);
          Tag := i;
        End;
      End
      Else
      Begin
        With AddMenu Do
        Begin
          Caption := GetCaption(i);
          tag := i;
        End;
      End;
    End;
  End;
End;

Procedure TRecentFileList.DoClick(Item: TMenuItem);
Begin
  If Assigned(FOnMenuClick) Then
    FOnMenuClick(Self, FList[Item.Tag])
End;

Procedure TRecentFileList.OnMenuItemClick(Sender: TObject);
Begin
  DoClick(TMenuItem(Sender));
End;

Function TRecentFileList.GetConfigString: String;
Var
  i                                     : integer;
Begin
  result := '';
  For i := 0 To FList.Count - 1 Do
    result := FList[i] + '|';
End;

Procedure TRecentFileList.SetConfigString(c: String);
Var
  s                                     : String;
  i                                     : integer;
Begin
  FList.Clear;
  Repeat
    i := Pos('|', c);
    s := Copy(c, 0, i - 1);
    If i = 0 Then s := c;
    s := Trim(s);
    If s <> '' Then FList.Add(s);
    Delete(c, 1, i)
  Until i = 0;
  CreateMenu;
End;

End.

